# coding=utf-8
"""
The Campaign Content API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/campaigns/content/
Schema: https://api.mailchimp.com/schema/3.0/Campaigns/Content/Collection.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class CampaignContent(BaseApi):
    """
    Manage the HTML, plain-text, and template content for your MailChimp
    campaigns.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(CampaignContent, self).__init__(*args, **kwargs)
        self.endpoint = 'campaigns'
        self.campaign_id = None


    def get(self, campaign_id, **queryparams):
        """
        Get the the HTML and plain-text content for a campaign.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.campaign_id = campaign_id
        return self._mc_client._get(url=self._build_path(campaign_id, 'content'), **queryparams)


    def update(self, campaign_id, data):
        """
        Set the content for a campaign.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        """
        self.campaign_id = campaign_id
        return self._mc_client._put(url=self._build_path(campaign_id, 'content'), data=data)



# coding=utf-8
"""
The E-commerce Stores Promo Rules API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/ecommerce/stores/promo-rules
"""
from __future__ import unicode_literals
from mailchimp3.baseapi import BaseApi

class StorePromoRules(BaseApi):
    """
    Promo Rules help you create promo codes for your campaigns. Promo Rules define generic information about promo
    codes like expiration time, start time, amount of discount being offered etc
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the Endpoint
        :param args:
        :param kwargs:
        """
        super(StorePromoRules, self).__init__(*args, **kwargs)
        self.endpoint = 'ecommerce/stores'
        self.store_id = None

    def create(self, store_id, data):
        """
        Add new promo rule to a store

        :param store_id: The store id
        :type store_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict'
        data = {
            "id": string*,
            "title": string,
            "description": string*,
            "starts_at": string,
            "ends_at": string,
            "amount": number*,
            "type": string*,
            "target": string*,
            "enabled": boolean,
            "created_at_foreign": string,
            "updated_at_foreign": string,
        }
        """
        self.store_id = store_id
        if 'id' not in data:
            raise KeyError('The promo rule must have an id')
        if 'description' not in data:
            raise KeyError('This promo rule must have a description')
        if 'amount' not in data:
            raise KeyError('This promo rule must have an amount')
        if 'target' not in data:
            raise KeyError('This promo rule must apply to a target (example per_item, total, or shipping')
        response = self._mc_client._post(url=self._build_path(store_id, 'promo-rules'), data=data)

        if response is not None:
            return response

    def all(self, store_id, get_all=False, **queryparams):
        """
        Get information about a store’s promo rules.

        :param store_id: The store's id
        :type store_id: `str`
        :param get_all:
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.store_id = store_id
        if get_all:
            return self._iterate(url=self._build_path(store_id, 'promo-rules'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(store_id, 'promo-rules'), **queryparams)

    def get(self, store_id, promo_rule_id, **queryparams):
        """
                Get information about a specific promo rule.

                :param store_id: The store's id
                :type store_id: `string`
                :param queryparams: The query string parameters
                queryparams['fields'] = []
                queryparams['exclude_fields'] = []
                queryparams['count'] = integer
                queryparams['offset'] = integer
                """
        self.store_id = store_id
        self.promo_rule_id = promo_rule_id
        return self._mc_client._get(url=self._build_path(store_id, 'promo-rules', promo_rule_id), **queryparams)


    def update(self, store_id, promo_rule_id, data):
        """
        Update a promo rule

        :param store_id: The store id
        :type :py:class:`str`
        :param promo_rule_id: The id for the promo rule of a store.
        :type :py:class:`str`
        :param data:
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "id": string,
            "title": string,
            "description": string,
            "starts_at": string,
            "ends_at": string,
            "amount": number,
            "type": string,
            "target": string,
            "enabled": boolean,
            "created_at_foreign": string,
            "updated_at_foreign": string,
        }
        """
        self.store_id = store_id
        self.promo_rule_id = promo_rule_id
        return self._mc_client._patch(url=self._build_path(store_id, 'promo-rules', promo_rule_id), data=data)

    def delete(self, store_id, promo_rule_id):
        """
        Delete a promo rule
        :param store_id: The store id
        :type :py:class:`str`
        :param promo_rule_id: The id for the promo rule of a store.
        :type :py:class:`str`
        """
        self.store_id=store_id
        self.promo_rule_id=promo_rule_id
        return self._mc_client._delete(url=self._build_path(store_id, 'promo-rules', promo_rule_id))


# coding=utf-8
"""
The E-commerce Store Product Images API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/ecommerce/stores/products/images/
Schema: https://api.mailchimp.com/schema/3.0/Ecommerce/Stores/Products/Images/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class StoreProductImages(BaseApi):
    """
    A Product Image represents a specific product image.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(StoreProductImages, self).__init__(*args, **kwargs)
        self.endpoint = 'ecommerce/stores'
        self.store_id = None
        self.product_id = None
        self.image_id = None


    def create(self, store_id, product_id, data):
        """
        Add a new image to the product.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param product_id: The id for the product of a store.
        :type product_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "id": string*,
            "url": string*
        }
        """
        self.store_id = store_id
        self.product_id = product_id
        if 'id' not in data:
            raise KeyError('The product image must have an id')
        if 'title' not in data:
            raise KeyError('The product image must have a url')
        response = self._mc_client._post(url=self._build_path(store_id, 'products', product_id, 'images'), data=data)
        if response is not None:
            self.image_id = response['id']
        else:
            self.image_id = None
        return response


    def all(self, store_id, product_id, get_all=False, **queryparams):
        """
        Get information about a product’s images.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param product_id: The id for the product of a store.
        :type product_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.store_id = store_id
        self.product_id = product_id
        self.image_id = None
        if get_all:
            return self._iterate(url=self._build_path(store_id, 'products', product_id, 'images'), **queryparams)
        else:
            return self._mc_client._post(url=self._build_path(store_id, 'products', product_id, 'images'), **queryparams)


    def get(self, store_id, product_id, image_id, **queryparams):
        """
        Get information about a specific product image.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param product_id: The id for the product of a store.
        :type product_id: :py:class:`str`
        :param image_id: The id for the product image.
        :type image_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.store_id = store_id
        self.product_id = product_id
        self.image_id = image_id
        return self._mc_client._post(
            url=self._build_path(store_id, 'products', product_id, 'images', image_id),
            **queryparams
        )


    def update(self, store_id, product_id, image_id, data):
        """
        Update a product image.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param product_id: The id for the product of a store.
        :type product_id: :py:class:`str`
        :param image_id: The id for the product image.
        :type image_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        """
        self.store_id = store_id
        self.product_id = product_id
        self.image_id = image_id
        return self._mc_client._patch(
            url=self._build_path(store_id, 'products', product_id, 'images', image_id),
            data=data
        )


    def delete(self, store_id, product_id, image_id):
        """
        Delete a product image.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param product_id: The id for the product of a store.
        :type product_id: :py:class:`str`
        :param image_id: The id for the product image.
        :type image_id: :py:class:`str`
        """
        self.store_id = store_id
        self.product_id = product_id
        self.image_id = image_id
        return self._mc_client._delete(url=self._build_path(store_id, 'products', product_id, 'images', image_id))



# coding=utf-8
"""
The E-commerce Store Carts endpoint API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/ecommerce/stores/carts/
Schema: https://api.mailchimp.com/schema/3.0/Ecommerce/Stores/Carts/Instance.json
"""
from __future__ import unicode_literals

import re

from mailchimp3.baseapi import BaseApi
from mailchimp3.entities.storecartlines import StoreCartLines


class StoreCarts(BaseApi):
    """
    Use Carts to represent unfinished e-commerce transactions. This can be
    used to create an Abandoned Cart workflow, or to save a consumer’s
    shopping cart pending a successful Order.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(StoreCarts, self).__init__(*args, **kwargs)
        self.endpoint = 'ecommerce/stores'
        self.store_id = None
        self.cart_id = None
        self.lines = StoreCartLines(self)


    def create(self, store_id, data):
        """
        Add a new cart to a store.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "id": string*,
            "customer": object*
            {
                "id": string*
            },
            "currency_code": string*,
            "order_total": number*,
            "lines": array*
            [
                {
                    "id": string*,
                    "product_id": string*,
                    "product_variant_id": string*,
                    "quantity": integer*,
                    "price": number*
                }
            ]
        }
        """
        self.store_id = store_id
        if 'id' not in data:
            raise KeyError('The cart must have an id')
        if 'customer' not in data:
            raise KeyError('The cart must have a customer')
        if 'id' not in data['customer']:
            raise KeyError('The cart customer must have an id')
        if 'currency_code' not in data:
            raise KeyError('The cart must have a currency_code')
        if not re.match(r"^[A-Z]{3}$", data['currency_code']):
            raise ValueError('The currency_code must be a valid 3-letter ISO 4217 currency code')
        if 'order_total' not in data:
            raise KeyError('The cart must have an order_total')
        if 'lines' not in data:
            raise KeyError('The cart must have at least one cart line')
        for line in data['lines']:
            if 'id' not in line:
                raise KeyError('Each cart line must have an id')
            if 'product_id' not in line:
                raise KeyError('Each cart line must have a product_id')
            if 'product_variant_id' not in line:
                raise KeyError('Each cart line must have a product_variant_id')
            if 'quantity' not in line:
                raise KeyError('Each cart line must have a quantity')
            if 'price' not in line:
                raise KeyError('Each cart line must have a price')
        response = self._mc_client._post(url=self._build_path(store_id, 'carts'), data=data)
        if response is not None:
            self.cart_id = response['id']
        else:
            self.cart_id = None
        return response


    def all(self, store_id, get_all=False, **queryparams):
        """
        Get information about a store’s carts.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.store_id = store_id
        self.cart_id = None
        if get_all:
            return self._iterate(url=self._build_path(store_id, 'carts'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(store_id, 'carts'), **queryparams)


    def get(self, store_id, cart_id, **queryparams):
        """
        Get information about a specific cart.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param cart_id: The id for the cart.
        :type cart_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.store_id = store_id
        self.cart_id = cart_id
        return self._mc_client._get(url=self._build_path(store_id, 'carts', cart_id), **queryparams)


    def update(self, store_id, cart_id, data):
        """
        Update a specific cart.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param cart_id: The id for the cart.
        :type cart_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        """
        self.store_id = store_id
        self.cart_id = cart_id
        return self._mc_client._patch(url=self._build_path(store_id, 'carts', cart_id), data=data)


    def delete(self, store_id, cart_id):
        """
        Delete a cart.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param cart_id: The id for the cart.
        :type cart_id: :py:class:`str`
        """
        self.store_id = store_id
        self.cart_id = cart_id
        return self._mc_client._delete(url=self._build_path(store_id, 'carts', cart_id))



# coding=utf-8
"""
The Search Members API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/search-campaigns/
Schema: https://api.mailchimp.com/schema/3.0/Lists/Members/Collection.json
Additional Data: http://kb.mailchimp.com/accounts/management/search-for-subscribers-and-campaigns-in-your-account
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class SearchCampaigns(BaseApi):
    """
    Search all of an account’s campaigns for the specified query terms.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(SearchCampaigns, self).__init__(*args, **kwargs)
        self.endpoint = 'search-campaigns'


    def get(self, **queryparams):
        """
        Search all campaigns for the specified query terms.

        :param queryparams: The query string parameters
        queryparams['fields'] = array
        queryparams['exclude_fields'] = array
        queryparams['query'] = string
        queryparams['snip_start'] = string
        queryparams['snip_end'] = string
        queryparams['offset'] = integer
        """
        return self._mc_client._get(url=self._build_path(), **queryparams)



# coding=utf-8
"""
The Campaigns API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/campaigns/
Schema: https://api.mailchimp.com/schema/3.0/Campaigns/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.entities.campaignactions import CampaignActions
from mailchimp3.entities.campaigncontent import CampaignContent
from mailchimp3.entities.campaignfeedback import CampaignFeedback
from mailchimp3.entities.campaignsendchecklist import CampaignSendChecklist
from mailchimp3.helpers import check_email


class Campaigns(BaseApi):
    """
    Campaigns are how you send emails to your MailChimp list. Use the
    Campaigns API calls to manage campaigns in your MailChimp account.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(Campaigns, self).__init__(*args, **kwargs)
        self.endpoint = 'campaigns'
        self.campaign_id = None
        self.actions = CampaignActions(self)
        self.content = CampaignContent(self)
        self.feedback = CampaignFeedback(self)
        self.send_checklist = CampaignSendChecklist(self)


    def create(self, data):
        """
        Create a new MailChimp campaign.

        The ValueError raised by an invalid type in data does not mention
        'absplit' as a potential value because the documentation indicates
        that the absplit type has been deprecated.

        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "recipients": object*
            {
                "list_id": string*
            },
            "variate_settings": object* (Required if type is "variate")
            {
                "subject_lines": string* (can be empty if subject_line in settings)
                "from_names": string* (can be empty if from_name in settings)
                "reply_to_addresses": string* (can be empty if reply_to in settings)
                "winner_criteria": string* (Must be one of "opens", "clicks", "total_revenue", or "manual")
            },
            "rss_opts": object* (Required if type is "rss")
            {
                "feed_url": string*,
                "frequency": string* (Must be one of "daily", "weekly", or "monthly")
            },
            "type": string* (Must be one of "regular", "plaintext", "rss", "variate", or "absplit")
        }
        """
        if 'recipients' not in data:
            raise KeyError('The campaign must have recipients')
        if 'list_id' not in data['recipients']:
            raise KeyError('The campaign recipients must have a list_id')
        if 'reply_to' in data.get('settings', {}):
            check_email(data['settings']['reply_to'])
        if 'type' not in data:
            raise KeyError('The campaign must have a type')
        if not data['type'] in ['regular', 'plaintext', 'rss', 'variate', 'abspilt']:
            raise ValueError('The campaign type must be one of "regular", "plaintext", "rss", or "variate"')
        if data['type'] == 'variate':
            if 'variate_settings' not in data:
                raise KeyError('The variate campaign must have variate_settings')
            if 'winner_criteria' not in data['variate_settings']:
                raise KeyError('The campaign variate_settings must have a winner_criteria')
            if data['variate_settings']['winner_criteria'] not in ['opens', 'clicks', 'total_revenue', 'manual']:
                raise ValueError('The campaign variate_settings '
                                 'winner_criteria must be one of "opens", "clicks", "total_revenue", or "manual"')
            if 'subject_lines' not in data['variate_settings'] and 'subject_line' not in data.get('settings'):
                raise KeyError('The campaign variate_settings must have a subject_lines')
            if 'from_names' not in data['variate_settings'] and 'from_name' not in data.get('settings'):
                raise KeyError('The campaign variate_settings must have a from_names')
            if 'reply_to_addresses' not in data['variate_settings'] and 'reply_to' not in data.get('settings'):
                raise KeyError('The campaign variate_settings must have a reply_to_addresses')
        if data['type'] == 'rss':
            if 'rss_opts' not in data:
                raise KeyError('The rss campaign must have rss_opts')
            if 'feed_url' not in data['rss_opts']:
                raise KeyError('The campaign rss_opts must have a feed_url')
            if not data['rss_opts']['frequency'] in ['daily', 'weekly', 'monthly']:
                raise ValueError('The rss_opts frequency must be one of "daily", "weekly", or "monthly"')
        response = self._mc_client._post(url=self._build_path(), data=data)
        if response is not None:
            self.campaign_id = response['id']
        else:
            self.campaign_id = None
        return response


    def all(self, get_all=False, **queryparams):
        """
        Get all campaigns in an account.

        .. note::
            The before_create_time, since_create_time, before_send_time, and
            since_send_time queryparams expect times to be listed in the ISO
            8601 format in UTC (ex. 2015-10-21T15:41:36+00:00).

        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        queryparams['type'] = []
        queryparams['status'] = []
        queryparams['before_send_time'] = string
        queryparams['since_send_time'] = string
        queryparams['before_create_time'] = string
        queryparams['since_create_time'] = string
        queryparams['list_id'] = string
        queryparams['folder_id'] = string
        queryparams['sort_field'] = string
        queryparams['sort_dir'] = string
        """
        self.campaign_id = None
        if get_all:
            return self._iterate(url=self._build_path(), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(), **queryparams)


    def get(self, campaign_id, **queryparams):
        """
        Get information about a specific campaign.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['sort_field'] = string
        queryparams['create_time'] = string
        """
        self.campaign_id = campaign_id
        return self._mc_client._get(url=self._build_path(campaign_id), **queryparams)


    def update(self, campaign_id, data):
        """
        Update some or all of the settings for a specific campaign.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "settings": object*
            {
                "subject_line": string*,
                "from_name": string*,
                "reply_to": string*
            },
        }
        """
        self.campaign_id = campaign_id
        if 'settings' not in data:
            raise KeyError('The campaign must have settings')
        if 'subject_line' not in data['settings']:
            raise KeyError('The campaign settings must have a subject_line')
        if 'from_name' not in data['settings']:
            raise KeyError('The campaign settings must have a from_name')
        if 'reply_to' not in data['settings']:
            raise KeyError('The campaign settings must have a reply_to')
        check_email(data['settings']['reply_to'])
        return self._mc_client._patch(url=self._build_path(campaign_id), data=data)


    def delete(self, campaign_id):
        """
        Remove a campaign from your MailChimp account.

        :param campaign_id: The unique id for the campaign.
        :type campaign_id: :py:class:`str`
        """
        self.campaign_id = campaign_id
        return self._mc_client._delete(url=self._build_path(campaign_id))



#!/usr/bin/env python
import os
from setuptools import setup, find_packages


README = os.path.join(os.path.dirname(__file__), 'README.rst')

# When running tests using tox, README.md is not found
try:
    with open(README) as file:
        long_description = file.read()
except Exception:
    long_description = ''


setup(
    name='mailchimp3',
    version='3.0.19',
    description='A python client for v3 of MailChimp API',
    long_description=long_description,
    url='https://github.com/charlesthk/python-mailchimp',
    author='Charles TISSIER',
    author_email='charles@vingtcinq.io',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
    keywords='mailchimp api v3 client wrapper',
    packages=find_packages(),
    install_requires=['requests>=2.7.0'],
    # test_suite='tests',
)



# coding=utf-8
"""
The List Interest Category Interests API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/lists/interest-categories/interests/
Schema: https://api.mailchimp.com/schema/3.0/Lists/Interests/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class ListInterestCategoryInterest(BaseApi):
    """
    Manage interests for a specific MailChimp list. Assign subscribers to
    interests to group them together. Interests are referred to as ‘group
    names’ in the MailChimp application.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ListInterestCategoryInterest, self).__init__(*args, **kwargs)
        self.endpoint = 'lists'
        self.list_id = None
        self.category_id = None
        self.interest_id = None


    def create(self, list_id, category_id, data):
        """
        Create a new interest or ‘group name’ for a specific category.

        The documentation lists only the name request body parameter so it is
        being documented and error-checked as if it were required based on the
        description of the method.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param category_id: The unique id for the interest category.
        :type category_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "name": string*
        }
        """
        self.list_id = list_id
        self.category_id = category_id
        if 'name' not in data:
            raise KeyError('The list interest category interest must have a name')
        response =  self._mc_client._post(
            url=self._build_path(list_id, 'interest-categories', category_id, 'interests'),
            data=data
        )
        if response is not None:
            self.interest_id = response['id']
        else:
            self.interest_id = None
        return response


    def all(self, list_id, category_id, get_all=False, **queryparams):
        """
        Get a list of this category’s interests.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param category_id: The unique id for the interest category.
        :type category_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.list_id = list_id
        self.category_id = category_id
        self.interest_id = None
        if get_all:
            return self._iterate(url=self._build_path(list_id, 'interest-categories', category_id, 'interests'), **queryparams)
        else:
            return self._mc_client._get(
                url=self._build_path(list_id, 'interest-categories', category_id, 'interests'),
                **queryparams
            )


    def get(self, list_id, category_id, interest_id, **queryparams):
        """
        Get interests or ‘group names’ for a specific category.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param category_id: The unique id for the interest category.
        :type category_id: :py:class:`str`
        :param interest_id: The specific interest or ‘group name’.
        :type interest_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.list_id = list_id
        self.category_id = category_id
        self.interest_id = interest_id
        return self._mc_client._get(
            url=self._build_path(list_id, 'interest-categories', category_id, 'interests', interest_id),
            **queryparams
        )


    def update(self, list_id, category_id, interest_id, data):
        """
        Update interests or ‘group names’ for a specific category.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param category_id: The unique id for the interest category.
        :type category_id: :py:class:`str`
        :param interest_id: The specific interest or ‘group name’.
        :type interest_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "name": string*
        }
        """
        self.list_id = list_id
        self.category_id = category_id
        self.interest_id = interest_id
        if 'name' not in data:
            raise KeyError('The list interest category interest must have a name')
        return self._mc_client._patch(
            url=self._build_path(list_id, 'interest-categories', category_id, 'interests', interest_id),
            data=data
        )


    def delete(self, list_id, category_id, interest_id):
        """
        Delete interests or group names in a specific category.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param category_id: The unique id for the interest category.
        :type category_id: :py:class:`str`
        :param interest_id: The specific interest or ‘group name’.
        :type interest_id: :py:class:`str`
        """
        self.list_id = list_id
        self.category_id = category_id
        self.interest_id = interest_id
        return self._mc_client._delete(
            url=self._build_path(list_id, 'interest-categories', category_id, 'interests', interest_id)
        )



# coding=utf-8
"""
The base API object that allows constructions of various endpoint paths
"""
from __future__ import unicode_literals
from itertools import chain

from mailchimp3.helpers import merge_results


class BaseApi(object):
    """
    Simple class to buid path for entities
    """

    def __init__(self, mc_client):
        """
        Initialize the class with you user_id and secret_key

        :param mc_client: The mailchimp client connection
        :type mc_client: :mod:`mailchimp3.mailchimpclient.MailChimpClient`
        """
        super(BaseApi, self).__init__()
        self._mc_client = mc_client
        self.endpoint = ''

    def _build_path(self, *args):
        """
        Build path with endpoint and args

        :param args: Tokens in the endpoint URL
        :type args: :py:class:`unicode`
        """
        return '/'.join(chain((self.endpoint,), map(str, args)))

    def _iterate(self, url, **queryparams):
        """
        Iterate over all pages for the given url. Feed in the result of self._build_path as the url.

        :param url: The url of the endpoint
        :type url: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        # fields as a query string parameter should be a string with
        # comma-separated substring values to pass along to
        # self._mc_client._get(). It should also contain total_items whenever
        # the parameter is employed, which is forced here.
        if not self._mc_client.enabled:
            return
        if 'fields' in queryparams:
            if 'total_items' not in queryparams['fields'].split(','):
                queryparams['fields'] += ',total_items'
        # Remove offset if provided in queryparams to avoid 'multiple values
        # for keyword argument' TypeError
        queryparams.pop("offset", None)

        # Fetch results from mailchimp, up to first count. If count is not
        # provided, return a count of 500. The maximum value supported by the
        # api is 1000, but such a large request can cause 504 errors. See:
        # https://github.com/VingtCinq/python-mailchimp/pull/207
        count = queryparams.pop("count", 500)
        result = self._mc_client._get(url=url, offset=0, count=count, **queryparams)
        total = result['total_items']
        # Fetch further results if necessary
        if total > count:
            for offset in range(1, int(total / count) + 1):
                result = merge_results(result, self._mc_client._get(
                    url=url,
                    offset=int(offset * count),
                    count=count,
                    **queryparams
                ))
            return result
        else:  # Further results not necessary
            return result



# coding=utf-8
"""
The Conversation Messages API endpoint

Note: This is a paid feature

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/conversations/messages/
Schema: https://api.mailchimp.com/schema/3.0/Conversations/Messages/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.helpers import check_email


class ConversationMessages(BaseApi):
    """
    Manage messages in a specific campaign conversation.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ConversationMessages, self).__init__(*args, **kwargs)
        self.endpoint = 'conversations'
        self.conversation_id = None
        self.message_id = None


    # Paid feature
    def create(self, conversation_id, data):
        """
        Post a new message to a conversation.

        :param conversation_id: The unique id for the conversation.
        :type conversation_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "from_email": string*,
            "read": boolean*
        }
        """
        self.conversation_id = conversation_id
        if 'from_email' not in data:
            raise KeyError('The conversation message must have a from_email')
        check_email(data['from_email'])
        if 'read' not in data:
            raise KeyError('The conversation message must have a read')
        if data['read'] not in [True, False]:
            raise TypeError('The conversation message read must be True or False')
        response =  self._mc_client._post(url=self._build_path(conversation_id, 'messages'), data=data)
        if response is not None:
            self.message_id = response['id']
        else:
            self.message_id = None
        return response


    # Paid feature
    def all(self, conversation_id, **queryparams):
        """
        Get messages from a specific conversation.

        This endpoint does not currently support count and offset, preventing
        it from having the get_all parameter that most all() methods have

        :param conversation_id: The unique id for the conversation.
        :type conversation_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = p[
        queryparams['is_read'] = string
        queryparams['before_timestamp'] = string
        queryparams['since_timestamp'] = string
        """
        self.conversation_id = conversation_id
        self.message_id = None
        return self._mc_client._get(url=self._build_path(conversation_id, 'messages'), **queryparams)


    # Paid feature
    def get(self, conversation_id, message_id, **queryparams):
        """
        Get an individual message in a conversation.

        :param conversation_id: The unique id for the conversation.
        :type conversation_id: :py:class:`str`
        :param message_id: The unique id for the conversation message.
        :type message_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.conversation_id = conversation_id
        self.message_id = message_id
        return self._mc_client._get(url=self._build_path(conversation_id, 'messages', message_id), **queryparams)



# coding=utf-8
"""
The List Segment Members API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/lists/segments/members/
Schema: https://api.mailchimp.com/schema/3.0/Lists/Members/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi
from mailchimp3.helpers import check_email, check_subscriber_hash


class ListSegmentMembers(BaseApi):
    """
    Manage list members in a saved segment.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(ListSegmentMembers, self).__init__(*args, **kwargs)
        self.endpoint = 'lists'
        self.list_id = None
        self.segment_id = None
        self.subscriber_hash = None


    def create(self, list_id, segment_id, data):
        """
        Add a member to a static segment.

        The documentation does not currently elaborate on the path or request
        body parameters. Looking at the example provided, it will be assumed
        that email_address and status are required request body parameters and
        they are documented and error-checked as such.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param segment_id: The unique id for the segment.
        :type segment_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "email_address": string*,
            "status": string* (Must be one of 'subscribed', 'unsubscribed', 'cleaned', or 'pending')
        }
        """
        self.list_id = list_id
        self.segment_id = segment_id
        if 'email_address' not in data:
            raise KeyError('The list segment member must have an email_address')
        check_email(data['email_address'])
        if 'status' not in data:
            raise KeyError('The list segment member must have a status')
        if data['status'] not in ['subscribed', 'unsubscribed', 'cleaned', 'pending']:
            raise ValueError('The list segment member status must be one of "subscribed", "unsubscribed", "cleaned" or'
                             '"pending"')
        response = self._mc_client._post(url=self._build_path(list_id, 'segments', segment_id, 'members'), data=data)
        if response is not None:
            self.subscriber_hash = response['id']
        else:
            self.subscriber_hash = None
        return response


    def all(self, list_id, segment_id, get_all=False, **queryparams):
        """
        Get information about members in a saved segment.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param segment_id: The unique id for the segment.
        :type segment_id: :py:class:`str`
        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.list_id = list_id
        self.segment_id = segment_id
        self.subscriber_hash = None
        if get_all:
            return self._iterate(url=self._build_path(list_id, 'segments', segment_id, 'members'), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(list_id, 'segments', segment_id, 'members'), **queryparams)


    def delete(self, list_id, segment_id, subscriber_hash):
        """
        Remove a member from the specified static segment.

        :param list_id: The unique id for the list.
        :type list_id: :py:class:`str`
        :param segment_id: The unique id for the segment.
        :type segment_id: :py:class:`str`
        :param subscriber_hash: The MD5 hash of the lowercase version of the
          list member’s email address.
        :type subscriber_hash: :py:class:`str`
        """
        subscriber_hash = check_subscriber_hash(subscriber_hash)
        self.list_id = list_id
        self.segment_id = segment_id
        self.subscriber_hash = subscriber_hash
        return self._mc_client._delete(
            url=self._build_path(list_id, 'segments', segment_id, 'members', subscriber_hash)
        )



# coding=utf-8
"""
The API Root endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/root/
Schema: https://api.mailchimp.com/schema/3.0/Root.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class Root(BaseApi):
    """
    The API root resource links to all other resources available in the API.
    Calling the root directory also returns details about the MailChimp user
    account.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(Root, self).__init__(*args, **kwargs)
        self.endpoint = ''


    def get(self, **queryparams):
        """
        Get links to all other resources available in the API.

        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        return self._mc_client._get(url=self._build_path(), **queryparams)



# coding=utf-8
"""
The Template Default Content API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/templates/default-content/
Schema: https://api.mailchimp.com/schema/3.0/Templates/Defaultcontent/Collection.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class TemplateDefaultContent(BaseApi):
    """
    Manage the default content for a MailChimp template.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(TemplateDefaultContent, self).__init__(*args, **kwargs)
        self.endpoint = 'templates'
        self.template_id = None


    def all(self, template_id, **queryparams):
        """
        Get the sections that you can edit in a template, including each
        section’s default content.

        :param template_id: The unique id for the template.
        :type template_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.template_id = template_id
        return self._mc_client._get(url=self._build_path(template_id, 'default-content'), **queryparams)



# coding=utf-8
"""
The Template Folders API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/template-folders/
Schema: https://api.mailchimp.com/schema/3.0/TemplateFolders/Instance.json
"""
from __future__ import unicode_literals

from mailchimp3.baseapi import BaseApi


class TemplateFolders(BaseApi):
    """
    Organize your templates using folders.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(TemplateFolders, self).__init__(*args, **kwargs)
        self.endpoint = 'template-folders'
        self.folder_id = None


    def create(self, data):
        """
        Create a new template folder.

        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "name": string*
        }
        """
        if 'name' not in data:
            raise KeyError('The template folder must have a name')
        response = self._mc_client._post(url=self._build_path(), data=data)
        if response is not None:
            self.folder_id = response['id']
        else:
            self.folder_id = None
        return response


    def all(self, get_all=False, **queryparams):
        """
        Get all folders used to organize templates.

        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.folder_id = None
        if get_all:
            return self._iterate(url=self._build_path(), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(), **queryparams)


    def get(self, folder_id, **queryparams):
        """
        Get information about a specific folder used to organize templates.

        :param folder_id: The unique id for the File Manager folder.
        :type folder_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.folder_id = folder_id
        return self._mc_client._get(url=self._build_path(folder_id), **queryparams)


    def update(self, folder_id, data):
        """
        Update a specific folder used to organize templates.

        :param folder_id: The unique id for the File Manager folder.
        :type folder_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "name": string*
        }
        """
        if 'name' not in data:
            raise KeyError('The template folder must have a name')
        self.folder_id = folder_id
        return self._mc_client._patch(url=self._build_path(folder_id), data=data)


    def delete(self, folder_id):
        """
        Delete a specific template folder, and mark all the templates in the
        folder as ‘unfiled’.

        :param folder_id: The unique id for the File Manager folder.
        :type folder_id: :py:class:`str`
        """
        self.folder_id = folder_id
        return self._mc_client._delete(url=self._build_path(folder_id))





# coding=utf-8
"""
The E-commerce Stores API endpoint

Documentation: http://developer.mailchimp.com/documentation/mailchimp/reference/ecommerce/stores/
Schema: https://api.mailchimp.com/schema/3.0/Ecommerce/Stores/Instance.json
"""
from __future__ import unicode_literals

import re

from mailchimp3.baseapi import BaseApi
from mailchimp3.entities.storecarts import StoreCarts
from mailchimp3.entities.storecustomers import StoreCustomers
from mailchimp3.entities.storeorders import StoreOrders
from mailchimp3.entities.storeproducts import StoreProducts


class Stores(BaseApi):
    """
    Connect your E-commerce Store to MailChimp to take advantage of powerful
    reporting and personalization features and to learn more about your
    customers.
    """
    def __init__(self, *args, **kwargs):
        """
        Initialize the endpoint
        """
        super(Stores, self).__init__(*args, **kwargs)
        self.endpoint = 'ecommerce/stores'
        self.store_id = None
        self.carts = StoreCarts(self)
        self.customers = StoreCustomers(self)
        self.orders = StoreOrders(self)
        self.products = StoreProducts(self)

    def create(self, data):
        """
        Add a new store to your MailChimp account.

        Error checking on the currency code verifies that it is in the correct
        three-letter, all-caps format as specified by ISO 4217 but does not
        check that it is a valid code as the list of valid codes changes over
        time.

        :param data: The request body parameters
        :type data: :py:class:`dict`
        data = {
            "id": string*,
            "list_id": string*,
            "name": string*,
            "currency_code": string*
        }
        """
        if 'id' not in data:
            raise KeyError('The store must have an id')
        if 'list_id' not in data:
            raise KeyError('The store must have a list_id')
        if 'name' not in data:
            raise KeyError('The store must have a name')
        if 'currency_code' not in data:
            raise KeyError('The store must have a currency_code')
        if not re.match(r"^[A-Z]{3}$", data['currency_code']):
            raise ValueError('The currency_code must be a valid 3-letter ISO 4217 currency code')
        response = self._mc_client._post(url=self._build_path(), data=data)
        if response is not None:
            self.store_id = response['id']
        else:
            self.store_id = None
        return response


    def all(self, get_all=False, **queryparams):
        """
        Get information about all stores in the account.

        :param get_all: Should the query get all results
        :type get_all: :py:class:`bool`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        queryparams['count'] = integer
        queryparams['offset'] = integer
        """
        self.store_id = None
        if get_all:
            return self._iterate(url=self._build_path(), **queryparams)
        else:
            return self._mc_client._get(url=self._build_path(), **queryparams)


    def get(self, store_id, **queryparams):
        """
        Get information about a specific store.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param queryparams: The query string parameters
        queryparams['fields'] = []
        queryparams['exclude_fields'] = []
        """
        self.store_id = store_id
        return self._mc_client._get(url=self._build_path(store_id), **queryparams)


    def update(self, store_id, data):
        """
        Update a store.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        :param data: The request body parameters
        :type data: :py:class:`dict`
        """
        self.store_id = store_id
        return self._mc_client._patch(url=self._build_path(store_id), data=data)


    def delete(self, store_id):
        """
        Delete a store. Deleting a store will also delete any associated
        subresources, including Customers, Orders, Products, and Carts.

        :param store_id: The store id.
        :type store_id: :py:class:`str`
        """
        self.store_id = store_id
        return self._mc_client._delete(url=self._build_path(store_id))
